@@ -2,6 +2,7 @@
 # #Import packages
 # library('EBImage')
 # library('jpeg')
+# library('tiff')
 # library('logging')
 # library('parallel')
 # library('PET')
@@ -9,28 +10,65 @@
 # 
 # source('Peaks.R')
 # source('Help.R')
-# 
 # setwd(z)
 
-.GITTER_VERSION = '1.0.2'
-.methods = c('kmeans', 'tophat')
-.pf = list('1536'=c(32,48),'768'=c(32,48),'384'=c(16,24),'96'=c(8,12))  
 
-gitter.demo <- function(eg='single'){
-  if(!eg %in% c('single', 'ref')) stop('Invalid example')
+.GITTER_VERSION = '1.0.3'
+
+.onAttach <- function(lib, pkg, ...) {
+#   writeLines(sprintf('gitter version %s - quantification of pinned microbial cultures\n', .GITTER_VERSION))
+#   writeLines('Copyright (C) 2014 Omar Wagih\n')
+#   writeLines("Type 'gitter.demo()' for a demo, '?gitter' for help ")
+#   writeLines("or see http://gitter.ccbr.utoronto.ca for more details")
+  packageStartupMessage(sprintf("gitter version %s - quantification of pinned microbial cultures\n
+Copyright (C) 2014 Omar Wagih\n
+Type 'gitter.demo()' for a demo, '?gitter' for help 
+or see http://gitter.ccbr.utoronto.ca for more details", .GITTER_VERSION))
+}
+
+.pf = list('1536'=c(32,48),'768'=c(32,48),'384'=c(16,24),'96'=c(8,12))
+
+.readImage <- function(file){
+  n = basename(file)
+  if(grepl('*.jpg$|*.jpeg$', file, ignore.case=T)){
+    return( readJPEG(file) )
+  }else if(grepl('*.tiff$', file, ignore.case=T)){
+    return( readTIFF(file) )
+  }else{
+    im = imageData( readImage(file) )
+    d = dim(im)
+    M = array(NA, dim=d[c(2,1,3)])
+    M[,,1] = t(im[,,1])
+    if(d[3] > 1){
+      M[,,2] = t(im[,,2])
+      M[,,3] = t(im[,,3])
+    }
+    return(M)
+  }
   
-  if(eg == 'single'){
+}
+
+
+
+gitter.demo <- function(eg=1){
+  if(!eg %in% c(1,2)) stop('Invalid example, please use 1 for a single image or 2 to process an image using a reference image')
+  
+  if(! interactive() ) stop('Unable to run demo through non-interactive interface!')
+  if(eg == 1){ # Process single image
     f = system.file("extdata", "sample.jpg", package="gitter")
-    dat = gitter(f)
-    p <- gitter.plot(dat, title=sprintf('gitter v%s single image example', .GITTER_VERSION))
+    dat = gitter(f, verbose='p')
+    p <- plot.gitter(dat, title=sprintf('gitter v%s single image example', .GITTER_VERSION))
     print(p)
     browseURL(f)
-    gitter.summary(dat)
+    browseURL(file.path(getwd(), paste0('gridded_', basename(f))))
+    summary.gitter(dat)
   }
-  if(eg == 'ref'){
+  if(eg == 2){ # Process using reference image
     f = system.file("extdata", "sample_dead.jpg", package="gitter")
     f.ref = system.file("extdata", "sample.jpg", package="gitter")
-    gitter.batch(f, f.ref)
+    gitter.batch(f, f.ref, verbose='p')
+    browseURL(f)
+    browseURL(file.path(getwd(), paste0('gridded_', basename(f))))
     warning(sprintf('NOTE: Output files were saved to working directory at %s', getwd()))
   }
 }
@@ -55,7 +93,8 @@ gitter.batch <- function(image.files, ref.image.file=NULL, verbose='l', ...){
   if(is.dir){
     image.files = image.files[1]
     loginfo('Reading images from directory: %s', image.files)
-    image.files = list.files(image.files, pattern='jpe?g$', full.names=T, ignore.case=T)
+    image.files = list.files(image.files, pattern='*.jpg$|*.jpeg$|*.tiff$', full.names=T, ignore.case=T)
+    image.files = image.files[!grepl('^gridded_', basename(image.files))] # ignore gridded images
     if(length(image.files) == 0) stop('No images with JPEG or JPG extension found. Images must be JPG format, please convert any non-JPG images to JPG')
   }
   
@@ -87,7 +126,8 @@ gitter.batch <- function(image.files, ref.image.file=NULL, verbose='l', ...){
   
   # Save failed plates
   if(length(failed.plates) > 0){
-    failed.plates = c('# gitter failed images', failed.plates)
+    failed.plates = c(sprintf('# gitter v%s failed images generated on %s', .GITTER_VERSION, format(Sys.time(), "%a %b %d %X %Y")),
+                      failed.plates)
     writeLines(failed.plates, failed.file)
   }
   #dats = lapply(image.files, gitter, ..., .params=params, .is.ref=F)
@@ -100,6 +140,7 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
                    inverse=F, verbose='l', contrast=NULL, fast=NULL, plot=F, grid.save=getwd(), 
                    dat.save=getwd(), .is.ref=F, .params=NULL){
   
+
   # Check if we have one number plate formats
   if(length(plate.format) == 1){
     t = as.character(plate.format)
@@ -121,6 +162,7 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
   } 
   if(grepl('^gridded', basename(image.file))) 
     warning('Detected gridded image as input')
+  
   if(!is.null(grid.save)){
     if(!file.info(grid.save)$isdir) 
       stop(sprintf('Invalid gridded directory "%s"', grid.save))
@@ -133,7 +175,6 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
   if(is.fast){
     if(fast < 1500 | fast > 4000) stop('Fast resize width must be between 1500-4000px')
   } 
-   
   
   expf = 1.5
   params = as.list(environment(), all=TRUE)
@@ -149,14 +190,14 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
   prog = verbose == 'p' 
   if(!prog) pb <- NULL
   if(prog){
-    cat(file.path(dirname(image.file), basename(image.file)), '\n')
+    cat('Processing', basename(image.file), '...\n')
     pb <- txtProgressBar(min = 0, max = 100, style = 3)
   }
   # Read image
   loginfo('Reading image from: %s', image.file)
   if(prog) setTxtProgressBar(pb, 5)
   
-  im = readJPEG(image.file)
+  im = .readImage(image.file)
   is.color = (length(dim(im)) == 3)
   
   if(is.fast){
@@ -176,7 +217,6 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
     im.grey = im
   }
   
-  
   if(autorotate){
     if(prog) setTxtProgressBar(pb, 14)
     loginfo('Autorotating image...')
@@ -259,6 +299,7 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
   
   # Window expansion factor
   d = round(w * expf)
+  d = .roundOdd(d)
   
   if(prog) setTxtProgressBar(pb, 75)
   #Pad image
@@ -278,10 +319,11 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
   im.grey = .unpadmatrix(im.pad, w)
   
   coords[,c('x','y','xl','xr','yt','yb')] = coords[,c('x','y','xl','xr','yt','yb')]-w
-  coords[coords<0] = 1
+  coords[,1:6][coords[,1:6] < 0] = 1
   coords[,c('xl', 'xr')][coords[,c('xl', 'xr')] > ncol(im.grey)] = ncol(im.grey)
   coords[,c('yt', 'yb')][coords[,c('yt', 'yb')] > nrow(im.grey)] = nrow(im.grey)
   
+  
   # Generate rows and columns to be bound to coords below
   rc = expand.grid(1:ncol, 1:nrow)[,c(2,1)]
   names(rc) = c('row', 'col')
@@ -291,7 +333,7 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
   elapsed = signif( (proc.time() - ptm)[['elapsed']], 5)
   
   # Rearrange
-  results = results[,c('row', 'col', 'size', 'circularity',
+  results = results[,c('row', 'col', 'size', 'circularity','flags',
                        'x', 'y', 'xl','xr', 'yt', 'yb')]
   
   class(results) = c('gitter', 'data.frame')
@@ -301,23 +343,25 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
   if(!is.null(grid.save) & !.is.ref){
     #imr = drawPeaks(peaks.c=cp.y$peaks, peaks.r=cp.x$peaks, imr)
     imr = .drawRect(coords[,3:6], im.grey)
+    #imr = im.grey
     save = file.path(grid.save, paste0('gridded_',basename(image.file)))
     loginfo('Saved gridded image to: %s', save)
     
     if(prog) setTxtProgressBar(pb, 93)
-    writeJPEG(imr, save,1)
+    #writeJPEG(imr, save,1)
+    writeJPEG(imr, save)
   }
   
   if(prog) setTxtProgressBar(pb, 98)
+  
   # Save dat file
   if(!is.null(dat.save) & !.is.ref){
     save = file.path(dat.save, paste0(basename(image.file), '.dat'))
     
-    results[[4]] = round(results[[4]], 4)
-    results = results[,1:4]
+    results$circularity = round(results$circularity, 4)
+    results = results[,1:5]
     
-    write.table(results, file=save, quote=F, sep='\t', row.names=F)
-    loginfo('Saved dat file to: %s', save)
+    results = .gitter.write(results, save)
   }
   
   # Garbage collector
@@ -337,26 +381,6 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
   return(results)
 }
 
-# .getKmeansImage <- function(im, coords){
-#   coords = coords[,c('xl', 'xr', 'yt', 'yb')]
-#   cent = .getMeans(im)
-#   ret = im
-#   t = matrix(T, nrow(im), ncol(im))
-#   for(i in 1:nrow(coords)){
-#     z = as.numeric(coords[i,])
-#     spot = im[z[3]:z[4],z[1]:z[2]]
-#     im.spot.vec = c(cent, spot)
-#     #cent = c(min(im.spot.vec), max(im.spot.vec))
-#     cl = kmeans(im.spot.vec, cent)
-#     cls = cl$cluster[-(1:2)] - 1
-#     spot.bw = matrix(cls, nrow=nrow(spot))
-#     ret[z[3]:z[4],z[1]:z[2]] = spot.bw
-#     t[z[3]:z[4],z[1]:z[2]] = F
-#   }
-#   
-#   ret[t] = ( ret[t] > 0.7 )+0
-#   return(ret)
-# }
 
 .threshold <- function(im.grey, nrow, ncol, fast=T, f=1000, pb){
   prog = !is.null(pb)
@@ -419,15 +443,42 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
   return(x)
 }
 
-.xl <- function(z, w){ m = which(z == min(z)); t = length(z) - m[length(m)]; if(t < w) t = w; return(t) }
-.xr <- function(z, w){ t = which.min(z); if(t < w) t = w; return(t) }
+.xl <- function(z, w){ 
+  m = which(z == min(z))
+  t = length(z) - m[length(m)]
+  if(t < w) t = w
+  return(t)
+}
+
+.xr <- function(z, w){ 
+  t = which.min(z) + 1
+  if(t < w) t = w
+  return(t)
+}
 .yt <- .xl
 .yb <- .xr
 
+
+.matBorder <- function(x){
+  list(l = x[1:nrow(x),1],
+       r = x[1:nrow(x),ncol(x)],
+       t = x[1,1:ncol(x)],
+       b = x[nrow(x),1:ncol(x)])
+}
+
+.spilled <- function(x, f){
+  z = .matBorder(x)
+  sum( z$l ) > nrow(x)*f | 
+    sum( z$r ) > nrow(x)*f | 
+    sum( z$t ) > ncol(x)*f | 
+    sum( z$b ) > ncol(x)*f
+}
+
 .fitRects <- function(coords, im.kmeans, d){
   
   # Minimum border for really small colonies
-  minb = d/3
+  # Remove any decimals
+  minb = round(d/3)
   
   ret = lapply(1:nrow(coords), function(i){
     # Center of spot
@@ -441,12 +492,19 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
     
     spot.bw = im.kmeans[rect[3]:rect[4], rect[1]:rect[2]]
     
+    rs = rowSums(spot.bw)
+    cs = colSums(spot.bw)
+    
+    # Crop bw spot and compute new size
+    x.rel = x - rect[1]
+    y.rel = y - rect[3]
+    
     if(cent.pixel == 0){
-      z = rep(minb*2, 4)
+      z = rep( minb*2, 4)
     }else{
       # Sum rows and cols, split sums in half 
-      sp.y = .splitHalf(rowSums(spot.bw))
-      sp.x = .splitHalf(colSums(spot.bw))
+      sp.y = .splitHalf(rs)
+      sp.x = .splitHalf(cs)
       # Get minimums relative to the spot and set minimums if too small
       z = c(.xl(sp.x$left, minb), 
             .xr(sp.x$right, minb),
@@ -454,21 +512,41 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
             .yb(sp.y$right, minb))
     }
     
-    # Crop bw spot and compute new size
-    x.rel = x - rect[1]
-    y.rel = y - rect[3]
+#     if(i == 48){
+#       print(z)
+#       print(c(x.rel, y.rel))
+#       
+#       par(mfrow=c(1,2))
+#       plot(cs, type='l')
+#       abline(v=c(x.rel-z[1], x.rel+z[2]), col='red')
+#       plot(rs, type='l')
+#       abline(v=c(y.rel-z[3], x.rel+z[4]), col='blue')
+#     }
     
     rect.rel = c(x.rel - z[1], x.rel + z[2], y.rel - z[3], y.rel + z[4])
     spot.bw = spot.bw[rect.rel[3]:rect.rel[4], rect.rel[1]:rect.rel[2]]
+    
+    spilled = .spilled(spot.bw, 0.2)
+    
     # Compute new spot relative to plate
     rect = c(x - z[1], x + z[2], y - z[3], y + z[4])
     
-    c(x, y, rect, sum(spot.bw), .circularity(spot.bw))
+    c(x, y, rect, sum(spot.bw), .circularity(spot.bw), spilled)
   })
   
   coords = matrix(simplify2array(ret), nrow(coords), byrow=T)
   coords = as.data.frame(coords)
-  names(coords) = c('x', 'y', 'xl', 'xr', 'yt', 'yb', 'size', 'circularity')
+  names(coords) = c('x', 'y', 'xl', 'xr', 'yt', 'yb', 'size', 'circularity', 'spill')
+  
+  flags = data.frame(spilled = (coords$spill == 1), 
+                     lowcirc = (coords$circularity < 0.6 & !is.nan(coords$circularity) & !is.na(coords$circularity)))
+  # 1: Colony spill over or edge interference, 2 = Low circularity
+  flag.id = c('S', 'C')
+  coords$flags = apply(flags, 1, function(r){
+    paste0(flag.id[r], collapse=',')
+  })
+  coords[,names(coords) != 'spill']
+  
   return(coords)
 }
 
@@ -486,20 +564,90 @@ gitter <- function(image.file=file.choose(), plate.format=c(32,48), remove.noise
 }
 
 
+.getFlags <- function(dat){
+  s = dat$size / median(dat$size)
+  c = dat$circularity
+  n = nrow(dat)
+  
+  fl = c()
+  # If 10% of colonies have colonies smaller than 0.1
+  if( (sum(s < 0.1) / n) > 0.1){
+    fl = append(fl, "1")
+  }
+  # If empty 
+  if( (sum(is.na(c) | c < 0.6) / n) > 0.1 ){
+    fl = append(fl, "2")
+  }
+  return(fl)
+}
 
-gitter.plot <- function(dat, title='', type='heatmap', low='turquoise', mid='black', high='yellow', 
+.flagMap = c("1"="high count of small colony sizes",
+             "2"="high count of low colony circularity")
+.warningPat = "# Warning possible misgridding: "
+
+
+.gitter.write <- function(dat, path){
+  hd = c(sprintf('# gitter v%s data file generated on %s', .GITTER_VERSION, format(Sys.time(), "%a %b %d %X %Y")))
+  id = .getFlags(dat)
+  id = id[id %in% names(.flagMap)]
+  fl = unname(.flagMap[id])
+  if(length(fl) > 0){
+    attr(dat, 'warnings') = fl
+    fl = sprintf(paste0(.warningPat, "%s"), paste0(fl, collapse=', '))
+    hd = append(hd, fl)
+  }
+  hd = append(hd, '# Flags: S - Colony spill or edge interference, C - Low colony circularity')
+  writeLines(hd, path)
+  cat('# ', file=path, append=T)
+  
+  suppressWarnings( write.table(dat, file=path, quote=F, sep='\t', row.names=F, col.names=T, append=T) )
+  loginfo('Saved dat file to: %s', path)
+  return(dat)
+}
+
+
+gitter.read <- function(path){
+  if(is.character(path)){
+    dat = read.table(path, stringsAsFactors=F, header=F, sep='\t')
+    names(dat) = c('row', 'col', 'size', 'circularity', 'flags')
+    
+    #Read first 5 lines
+    con  <- file(path, open = "r")
+    i = 1
+    while (length(line <- readLines(con, n = 1, warn = FALSE)) > 0) {
+      if(i > 5) break;
+      if(grepl(pattern=.warningPat, line)){
+        line = gsub(.warningPat, replacement="", line)
+        x = strsplit(line, ", ")[[1]]
+        x = x[x %in% .flagMap]
+        attr(dat, "warnings") = x
+      }
+    } 
+    close(con)
+  }else if(!is.data.frame(path)){
+    stop('Please enter the filepath to a gitter data file')
+  }
+  class(dat) = c('gitter', 'data.frame')
+  return(dat)
+}
+
+plate.warnings <- function(dat){
+  if(!is.data.frame(dat) & ! 'gitter' %in% class(dat)) stop('Argument must be a gitter data object')
+  return( attr(dat, 'warnings') )
+}
+
+plot.gitter <- function(x, title='', type='heatmap', low='turquoise', mid='black', high='yellow', 
                         show.text=F, text.color='white', norm=T, 
-                        show.circ=T, circ.cutoff=0.6, circ.color='white'){
-  
-  
-  if(is.character(dat)) dat = read.table(dat, stringsAsFactors=F, header=T)
-  if(!is.data.frame(dat) & ! 'gitter' %in% class(dat)) stop('Data must be a data path or data frame generated by gitter')
+                        show.flags=T, flag.color='white', ...){
+  dat = x
+#   if(is.character(dat)) dat = read.table(dat, stringsAsFactors=F, header=T)
+  if(!is.data.frame(dat) & ! 'gitter' %in% class(dat)) stop('Argument must be a gitter data object')
   if(!type %in% c('heatmap', 'bubble')) stop('Invalid plot type. Use "heatmap" or "bubble"')
   
-  if(length(dat) > 4 | length(dat) < 3) stop('Invalid number of columns for dat file')
+  if(length(dat) > 5 | length(dat) < 3) stop('Invalid number of columns for dat file')
   
   names(dat) = c('r', 'c', 's')
-  dat.cs = NULL
+  dat.cs = s = NULL
   
   r = max(dat$r)
   c = max(dat$c)
@@ -517,16 +665,13 @@ gitter.plot <- function(dat, title='', type='heatmap', low='turquoise', mid='bla
     m = mean(dat$s[z[1]:z[2]], na.rm=T)
   }
   
-  if(length(dat) == 4){
-    names(dat)[4] = 'circ'
-    dat.cs = dat[dat$circ < circ.cutoff & !is.na(dat$circ),] 
+  if(length(dat) == 5){
+    names(dat)[5] = 'flags'
+    dat.cs = dat[dat$flags != "",] 
   }
   
   # Round data to 2
   dat$s = round(dat$s, 2)
-  
-  # For R CMD CHECK
-  s = NULL
   if(type == 'heatmap'){
     p <- ggplot(dat, aes(x = c, y = r, fill = s)) + 
       geom_tile(color='black') +
@@ -549,23 +694,35 @@ gitter.plot <- function(dat, title='', type='heatmap', low='turquoise', mid='bla
       theme_bw() +
       theme(title=element_text(size=14,face="bold"))
   }
-  if(show.circ & !is.null(dat.cs) & nrow(dat.cs) != 0){
-    p = p + geom_point(aes(x=c, y=r), data=dat.cs, color=circ.color, size=1)
+  if(show.flags & !is.null(dat.cs) & nrow(dat.cs) != 0){
+    p = p + geom_point(aes(x=c, y=r), data=dat.cs, color=flag.color, size=1)
   }
   
   if(show.text) p = p + geom_text(color = text.color, aes(label=s), size=3)
   return(p)
 }
 
-gitter.summary <- function(d){
+summary.gitter <- function(object, ...){
+  d = object
   pf = attr(d, 'format')
   call = attr(d, 'call')
-  writeLines(sprintf('###########################\n# gitter V%s data file #\n###########################', .GITTER_VERSION))
-  writeLines(sprintf('Function call: %s', deparse(call)))
+  if(is.null(call)){
+    call = "not available"
+  }else{
+    call = deparse(call)
+  }
+  
+  writeLines(sprintf('# gitter v%s data file #', .GITTER_VERSION))
+  writeLines(sprintf('Function call: %s', call))
   writeLines(sprintf('Elapsed time: %s secs', attr(d, 'elapsed')))
   writeLines(sprintf('Plate format: %s x %s (%s)', pf[1], pf[2], prod(pf)))
   writeLines('Colony size statistics:')
   print(summary(d[[3]]))
-  writeLines('Dat file (showing 6 rows):')
+  writeLines('Dat file (first 6 rows):')
   print(head(d))
+  w = attr(object, 'warnings')
+  if(! is.null(w)){
+    writeLines('Plate warnings:')
+    writeLines(paste0(w, collapse=', '))
+  }
 }
